// Chad Nelson

#include "Menu.h"

using namespace corona;

Menu::Menu() {
	max_frames = 210;
	frame_count = 0;
	done_flag = false;
	bw = load("chaddington.bmp");
	color = load("chaddington_color.bmp");
}

bool Menu::isDone() {
	return frame_count > max_frames;
}

void Menu::draw() {
	frame_count++;
	if (frame_count == max_frames) done_flag = true;
	if (frame_count >= max_frames) return;

	float alpha = 1.0;

	if (frame_count < 15) {
		alpha = (float) frame_count / 15.0;
		// Fade in BW image
		drawImage(bw, alpha, 0);
	} else if (frame_count < 60) {
		// Display BW image
		drawImage(bw, 1.0, 0);
	} else if (frame_count < 90) {
		alpha = (float) (frame_count - 60) / 30.0;
		// Fade in Color image ontop of BW
		drawImage(bw, 1.0, -0.5);
		drawImage(color, alpha, 0.5);
	} else if (frame_count < 120) {
		// Display Color image
		drawImage(color, 1.0, 0);
	} else if (frame_count < 195) {
		// Fade out Color image
		alpha = (float) -(frame_count - 150) / 15.0;
		drawImage(color, alpha, 0);
	}
	
}

void Menu::drawImage(int i, float alpha, float z) {
	glEnable(GL_TEXTURE_2D);
	glColor4f(1.0, 1.0, 1.0, alpha);
	glBindTexture(GL_TEXTURE_2D, i);
	glEnable(GL_BLEND);
	glBegin(GL_POLYGON);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(0, 0, z);

		glTexCoord2f(0.0, 1.0);
		glVertex3f(0, glutGet(GLUT_WINDOW_HEIGHT), z);

		glTexCoord2f(1.0, 1.0);
		glVertex3f(glutGet(GLUT_WINDOW_WIDTH), glutGet(GLUT_WINDOW_HEIGHT), z);

		glTexCoord2f(1.0, 0.0);
		glVertex3f(glutGet(GLUT_WINDOW_WIDTH), 0, z);
	glEnd();

	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
}


///// THIS FUNCTION IS AN EXAMPLE FROM THE GOODIES FOLDER ///////
int Menu::load(char *filename)
{
	Image* image;
	GLuint texname;
	image=OpenImage(filename, PF_R8G8B8A8);
	if(!image)
	{
		printf("Error loading image file: %s\n", filename);
		return 0;
	}

	int width  = image->getWidth();
	int height = image->getHeight();
	image=FlipImage(image, CA_X);
	
	// Generate texture names
	glGenTextures(1, &texname);
	
	// bind a named texture to a texturing target
	glBindTexture(GL_TEXTURE_2D, texname);
	
	// set pixel storage modes
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	// build a 2D mipmap - builds a series of prefiltered 2D texture maps of decreasing resolutions
	gluBuild2DMipmaps(GL_TEXTURE_2D, 4, width, height, GL_RGBA, GL_UNSIGNED_BYTE, image->getPixels());


	// set texture parameters
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	//use GL_MODULATE for standard texturing
	//GL_DECAL for unlightedness
	//GL_BLEND for interaction with a background color using alpha channel of image
	//GL_REPLACE to completely disregard lighting and replace it with the actual colors from the texture image
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	float col[]={1, 1, 1, 1};
	//I think this sets the background color of the object for GL_BLEND
	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, col);

	delete image;
		printf("Loading tex %i\n", texname);
	return texname;
}