// Chad Nelson

#include "Snake.h"

Snake::Snake() {
	reset();
}

void Snake::reset() {
	setLevel(1);
}

void Snake::setLevel(int l) {
	if (l < 0 || l > 5) return;
	snake_coordinates.clear();
	next_pos = new Point2D(0, 0);
	offset = 0.0;
	grow = 0;


	switch(l) {
		case 1:
			velocity = new Point2D(0, 1);
			direction = 0;
			x = 5.0;
			y = 5.0;
			snake_coordinates.push_back(new Point2D(5,5));
			snake_coordinates.push_back(new Point2D(5,4));
			snake_coordinates.push_back(new Point2D(5,3));
			snake_coordinates.push_back(new Point2D(5,2));
			snake_coordinates.push_back(new Point2D(5,1));
			break;
		case 2:
			velocity = new Point2D(0, -1);
			direction = 2;
			x = 2.0;
			y = 16.0;
			snake_coordinates.push_back(new Point2D(2,16));
			snake_coordinates.push_back(new Point2D(2,17));
			snake_coordinates.push_back(new Point2D(2,18));
			snake_coordinates.push_back(new Point2D(2,19));
			snake_coordinates.push_back(new Point2D(2,20));
			break;
		case 3:
			velocity = new Point2D(1, 0);
			direction = 3;
			x = 6.0;
			y = 38.0;
			snake_coordinates.push_back(new Point2D(6,38));
			snake_coordinates.push_back(new Point2D(5,38));
			snake_coordinates.push_back(new Point2D(4,38));
			snake_coordinates.push_back(new Point2D(3,38));
			snake_coordinates.push_back(new Point2D(2,38));
			break;
		case 4:
			velocity = new Point2D(-1, 0);
			direction = 1;
			x = 24.0;
			y = 2.0;
			snake_coordinates.push_back(new Point2D(24,2));
			snake_coordinates.push_back(new Point2D(25,2));
			snake_coordinates.push_back(new Point2D(26,2));
			snake_coordinates.push_back(new Point2D(27,2));
			snake_coordinates.push_back(new Point2D(28,2));
			break;
		case 5:
			velocity = new Point2D(0, 1);
			direction = 0;
			x = 2.0;
			y = 2.0;
			snake_coordinates.push_back(new Point2D(2,4));
			snake_coordinates.push_back(new Point2D(2,3));
			snake_coordinates.push_back(new Point2D(2,2));
			break;
	}
	
	next_pos->x = velocity->x + snake_coordinates.front()->x;
	next_pos->y = velocity->y + snake_coordinates.front()->y;
}

void Snake::draw() {
	list<Point2D *>::iterator it;
	Point2D *prev = snake_coordinates.front();
	bool follow = false;
	for(it = snake_coordinates.begin(); it != snake_coordinates.end(); ++it) {
		Point2D *p = *it;
		float px, py;

		glPushMatrix();
			// Set color & position
			if (it == snake_coordinates.begin()) {
				glColor3f(0.0, 1.0, 0.0);
				x = p->x + 0.5 + (velocity->x * offset);
				y = p->y + 0.5 + (velocity->y * offset);
					
				glTranslatef(x * TILE_SIZE, TILE_SIZE / 2.0, y * TILE_SIZE);
				glutSolidSphere(TILE_SIZE / 2.0, 20, 20);
			} else {
				glColor3f(1.0, 0.0, 0.0);
				px = p->x + 0.5 + ((prev->x - p->x) * offset);
				py = p->y + 0.5 + ((prev->y - p->y) * offset);

				if (!follow) {
					x_follow = px;
					y_follow = py;
				}
				follow = true;

				glTranslatef(px * TILE_SIZE, TILE_SIZE / 2.0, py * TILE_SIZE);
				glutSolidSphere(TILE_SIZE / 2.5, 20, 20);
			}
		glPopMatrix();

		prev = p;
	}

	
	if (grow > 0) {
		glPushMatrix();
			glColor3f(1.0, 0.0, 0.0);
			glTranslatef((prev->x + 0.5) * TILE_SIZE, TILE_SIZE / 2.0, (prev->y + 0.5) * TILE_SIZE);
			glutSolidSphere(TILE_SIZE / 2.5, 20, 20);
		glPopMatrix();
	}
}

void Snake::move() {
	Point2D *front = snake_coordinates.front();
	Point2D *new_position = new Point2D(front->x + velocity->x, front->y + velocity->y);
	if (grow <= 0) snake_coordinates.pop_back();
	if (grow > 0) grow--;
	snake_coordinates.push_front(new_position);

	next_pos->x = velocity->x + snake_coordinates.front()->x;
	next_pos->y = velocity->y + snake_coordinates.front()->y;
}

void Snake::changeDirection(int i) {
	direction = (direction + i + 4) % 4;
	velocity->x = 0;
	velocity->y = 0;
	switch (direction) {
		case 0:
			velocity->y = 1;
			break;
		case 1:
			velocity->x = -1;
			break;
		case 2:
			velocity->y = -1;
			break;
		case 3:
			velocity->x = 1;
			break;
	}

	next_pos->x = velocity->x + snake_coordinates.front()->x;
	next_pos->y = velocity->y + snake_coordinates.front()->y;
}