// Chad Nelson

#include "main.h"

Game *game;

/************** main *****************/
int main(int argc, char** argv) {
	// Seed random number generator
	srand ( time(NULL) );

	// Initialize the opengl window
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(640, 480);
	glutInitWindowPosition(100, 100);

	// Create the opengl window
	glutCreateWindow("CMN Snake3D");
    
    // init() consists of all the user defined initialization functions & variables
	init();

	// Register handlers
	glutDisplayFunc(redraw);
	glutKeyboardFunc(keyboardHandler);
	glutSpecialFunc(arrowHandler);
	glutReshapeFunc(resizeHandler);
	glutMenuStateFunc(menuStatusHandler);
	glutTimerFunc(FRAME_DELAY, timerHandler, 0);

	// Enter the opengl event processing loop
	glutMainLoop();

	return 0;
}



/************** init ***************/
void init(void) {
	resizeHandler(640, 480);
    glClearColor(0.0, 0.0, 0.0, 0.0);
	
	// Depth test
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);

	// Enable lighting
	glEnable(GL_LIGHTING);
	glEnable(GL_NORMALIZE);
    
	// Ambient lighting
	float ambient[] = { 0.3f, 0.3f, 0.3f, 1.0f };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
	
	// Create the sun
	glEnable(GL_LIGHT0);

	// Enable colored materials
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Enable blending for reflections
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	// Create Menu System
	int difficulty = glutCreateMenu(menuHandler);
	glutAddMenuEntry("Novice (1)", NOVICE);
	glutAddMenuEntry("Difficult (2)", DIFFICULT);
	glutAddMenuEntry("Insane (3)", INSANE);
	int camera = glutCreateMenu(menuHandler);
	glutAddMenuEntry("Camera Follow (F)", CAMERA_FOLLOW);
	glutAddMenuEntry("Camera Top (T)", CAMERA_TOP);
	glutAddMenuEntry("Zoom In (-)", ZOOM_IN);
	glutAddMenuEntry("Zoom Out (+)", ZOOM_OUT);
	int menu = glutCreateMenu(menuHandler);
	glutAddMenuEntry("New Game (F2)", NEW_GAME);
	glutAddSubMenu("Difficulty ->", difficulty);
	glutAddSubMenu("Camera ->", camera);
	glutAddMenuEntry("Toggle Minimap (H)", TOGGLE_MINIMAP);
	glutAddMenuEntry("Quit (Esc)", QUIT);
	glutAttachMenu(GLUT_RIGHT_BUTTON);

	// Create Game
	game = new Game();
}



/************** redraw ************/
void redraw(void) {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	// Setup for drawing 3D Scene
	resizeHandler(glutGet(GLUT_WINDOW_WIDTH), glutGet(GLUT_WINDOW_HEIGHT));
	// Draw 3D Scene
	game->draw();
	
	// Setup for drawing 2D Overlay
	glClear(GL_DEPTH_BUFFER_BIT);	// draw ontop of 3D scene
	
	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	gluOrtho2D(0, glutGet(GLUT_WINDOW_WIDTH), 0, glutGet(GLUT_WINDOW_HEIGHT));
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH);

	game->draw2D();
	
	glEnable(GL_DEPTH);
	glEnable(GL_LIGHTING);

	glFlush();
	glutSwapBuffers();
}



/************** timer ***********/
void timerHandler(int i) {
	glutTimerFunc(FRAME_DELAY, timerHandler, 0);

	// Update Game State
	game->update();

	// Redraw	
	glutPostRedisplay();
}



/************** resize ***********/
void resizeHandler(int w, int h) {
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0, (float) w / h, 0.5, 1000.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}



/************** Keyboard Handler **********/
void keyboardHandler(unsigned char key, int x, int y) {
	switch(key) {
		case 'f':
		case 'F':
			menuHandler(CAMERA_FOLLOW);
			break;
		case 't':
		case 'T':
			menuHandler(CAMERA_TOP);
			break;
		case 'p':
		case 'P':
			menuHandler(PAUSE);
			break;
		case 'm':
		case 'M':
			menuHandler(TOGGLE_MINIMAP);
			break;
		case '1':
			menuHandler(NOVICE);
			break;
		case '2':
			menuHandler(DIFFICULT);
			break;
		case '3':
			menuHandler(INSANE);
			break;
		case 27:
			menuHandler(QUIT);
			break;
		case '=':
		case '+':
			menuHandler(ZOOM_IN);
			break;
		case '-':
		case '_':
			menuHandler(ZOOM_OUT);
			break;
		case 32:
			game->pause = false;
			break;

		// REMOVE THESE TWO CASES - they allow cheating!
		case 'k':
			game->setLevel(game->board->level - 1);
			break;
		case 'l':
			game->setLevel(game->board->level + 1);
			break;
	}
}

/************** Keyboard (Special) Handler **********/
void arrowHandler(int key, int x, int y) {
	switch (key) {
		// Direction key pressed
		case GLUT_KEY_LEFT:
			game->pause = false;
			game->turn = TURN_LEFT;
			break;
		case GLUT_KEY_RIGHT:
			game->pause = false;
			game->turn = TURN_RIGHT;
			break;
		case GLUT_KEY_UP:
		case GLUT_KEY_DOWN:
			game->pause = false;
			game->turn = TURN_NONE;
			break;
			break;
		// Menu Shortcuts
		case GLUT_KEY_F2:
			menuHandler(NEW_GAME);
			break;
	}
}



/************** Menu Handler **********/
void menuHandler(int item) {
	switch(item) {
		case QUIT:
			exit(0);
			break;
		case PAUSE:
			game->pause = true;
			break;

		// Alter camera position
		case ZOOM_IN:
			game->zoom = max(3, game->zoom - 1);
			break;
		case ZOOM_OUT:
			game->zoom = min(12, game->zoom + 1);
			break;
		case CAMERA_TOP:
		case CAMERA_FOLLOW:
		case CAMERA_CINE:
			game->camera = item;
			break;

		// Start a new game
		case NEW_GAME:
			game->reset();
			break;

		// Change Difficulty
		case NOVICE:
		case DIFFICULT:
		case INSANE:
			game->changeDifficulty(item);
			break;

		// Toggle minimap
		case TOGGLE_MINIMAP:
			game->minimap = (game->minimap + 1) % 3;
			break;
	}
	glutPostRedisplay();
}

void menuStatusHandler(int state) {
	game->menu_in_use = state;
}