#include "render.h"

using namespace std;

// Default constructor
// used for initializing any user defined variables
Render::Render()
{
	// specify a default location for the camera
	eye[0] = 0.0f;
	eye[1] = 0.0f;
	eye[2] = 40.0f;

	// specify default values to the rotational values in the transformation matrix
	rot[0] = 0.0f;
	rot[1] = 0.0f;
	rot[2] = 0.0f;

	LEFT = 0;
	MIDDLE = 1;
	RIGHT = 2;
	mButton = -1;
}

Render::~Render()
{


}

void Render::init(void)
{
	// clear the background to the color values specified in the parentheses
	glClearColor (0.0, 0.0, 0.0, 0.0);

	// this is a 3D scene. so, clear the depth
	glClearDepth(1.0);

	// I need depth. enable depth
	glEnable(GL_DEPTH_TEST);

	glShadeModel (GL_SMOOTH);

	// Initialize lighting
	GLfloat LightAmbient[] = { 0.2f, 0.2f, 0.2f, 1.0f};
	GLfloat LightDiffuse[] = {1.0f, 1.0f, 1.0f, 1.0f};
	glLightfv (GL_LIGHT0, GL_AMBIENT, LightAmbient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, LightDiffuse);
	glEnable (GL_LIGHT0);
	glEnable (GL_LIGHTING);
	// not only did i initialize lighting, i also enabled material color
	// that means, when i have lighting, i can give the material a color just by altering glColor3f and
	glEnable (GL_COLOR_MATERIAL);
	glColorMaterial (GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	
	tex1 = loadtexture.load("images/laughingCat.jpg");
	tex2 = loadtexture.load("images/simptrix.pcx");
	tex3 = loadtexture.load("images/HelloWorld.bmp");
	tex4 = loadtexture.load("images/babytux.png");
}

void Render::reshape(int w, int h)
{
	glViewport(0,0,(GLsizei) w, (GLsizei) h);
	Wx = w;
	Wy = h;
}

void Render::mouseClick(int button, int state, int x, int y)
{
	if (state == GLUT_DOWN) 
	{
		mOldX = x;
		mOldY = y;
        switch (button)  
		{
			case GLUT_LEFT_BUTTON:
				mButton = LEFT;
				break;
			case GLUT_MIDDLE_BUTTON: 
				mButton = MIDDLE; 
				break;
			case GLUT_RIGHT_BUTTON:
				mButton = RIGHT;
				break;
			default:
				break;
		}
	}
	else if (state == GLUT_UP)
	{
		mButton = -1;
	}
}

void Render::mouseMove(int x, int y)
{
	if (mButton == LEFT) 
	{
		rot[0] -= ((mOldY - y) * 180.0f) / 1000.0f;
		rot[1] -= ((mOldX - x) * 180.0f) / 1000.0f;
		clamp(rot[0], rot[1], rot[2]);
	}
	else if (mButton == MIDDLE) 
	{
		eye[2] -= ((mOldY - y) * 180.0f) / 500.0f;
		clamp(rot[0], rot[1], rot[2]);
	}
	else if (mButton == RIGHT)
	{
		eye[0] += ((mOldX - x) * 180.0f) / 1000.0f;
		eye[1] -= ((mOldY - y) * 180.0f) / 1000.0f;
		clamp(rot[0], rot[1], rot[2]);
	}	 
	mOldX = x; 
	mOldY = y;

	// i changed values in the modelview matrix. so
	// update my display and the objects on the screens
	glutPostRedisplay();
}

void Render::keyPos(unsigned char key, int x, int y)
{
	switch (key)
	{
		default:
			break;
	    case 27:	// this is for the 'Esc' key on the keyboard, quit the program when the esc key is pressed
			exit (0);
	}
}

void Render::display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Specifies which matrix stack is the target for subsequent matrix operations
	// In this example, the projection matrix is set to perspective projection matrix stack
	glMatrixMode(GL_PROJECTION);
	// all matrix values from previous frames set to identity
	glLoadIdentity();
	
	// perspective projection loaded with new values for Wx and Wy updated
	gluPerspective(45, (GLfloat) Wx/(GLfloat) Wy, 1.0, 10000.0);
	// use glOrtho for a 3D orthogonal projection
	//glOrtho(-100, 100, -100, 100, -100, 100);

	// Applies subsequent matrix operations to the modelview matrix stack.
	glMatrixMode(GL_MODELVIEW);
	// Clears all the previously loaded values in the modelview matrix
	glLoadIdentity();
	// Now, add a bunch of transformation operations
	glTranslatef(-eye[0], -eye[1], -eye[2]);
    glRotatef(rot[0], 1.0f, 0.0f, 0.0f);
    glRotatef(rot[1], 0.0f, 1.0f, 0.0f);
    glRotatef(rot[2], 0.0f, 0.0f, 1.0f);

	// finally, draw the textured box
	drawTextureBox();

	// this allows opengl to wait for the draw buffer to be ready in the background for the next frame
	// therefore, while the current buffer is being drawn in the current frame, a buffer is set ready to draw on frame+1
	// this call is effective only when GL_DOUBLE is enabled in glutInitDisplayMode in the main function
	// It is recommended to use glutSwapBuffers and double buffering always
	glutSwapBuffers();
}

void Render::drawTextureBox(void)
{
	int i, j;
	
	// draw floor - grid of lines
	// First, remember to switch off lighting
	glDisable(GL_LIGHTING);
	// Assign a color to the floor
	glColor3f(0.0, 1.0, 0.0);
	// loop through so both horizontal and vertical lines are drawn on the floor
	for(i=-15; i<=15; i++)
	{
		glBegin(GL_LINES);
			glVertex3f(-i, -15, -15);
			glVertex3f(-i, -15, 15);
		glEnd();
		
		glBegin(GL_LINES);
			glVertex3f(-15, -15, -i);
			glVertex3f(15, -15, -i);
		glEnd();
	}
	// Turn lighting back on
	glEnable(GL_LIGHTING);
	
	// Assign a different color for the polygon to be textured
	glColor3f(1.0, 1.0, 1.0);
	// Enable texturing now
	glEnable(GL_TEXTURE_2D);


	// Draw Front wall
	glBindTexture(GL_TEXTURE_2D, tex1);
	// Bind the texture initialized in the init() function, here we are initializing tex1
	// Texturing can be made only on a polygon. So, glBegin(GL_POLYGON) is called
	// glTexCoord2f() is used for texture coordinates. They go from 0 to 1.
	// glVertex3f() is the place at which the polygon is initialized. Also, the texture coordinate (0,0) is bound to this vertex
	// glNormal3f() call is used to create a normal vector. Used for lighting purposes

	glBegin(GL_POLYGON);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(-15.0, -15.0, -15.0);
		glNormal3f(0, 0, 1);

		glTexCoord2f(1.0, 0.0);
		glVertex3f(15.0, -15.0, -15.0);
		glNormal3f(0, 0, 1);

		glTexCoord2f(1.0, 1.0);
		glVertex3f(15.0, 15.0, -15.0);
		glNormal3f(0, 0, 1);

		glTexCoord2f(0.0, 1.0);
		glVertex3f(-15.0, 15.0, -15.0);
		glNormal3f(0, 0, 1);
	glEnd();


	// Draw Right wall
	glBindTexture(GL_TEXTURE_2D, tex2);
	glBegin(GL_POLYGON);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(15.0, -15.0, -15.0);
		glNormal3f(-1, 0, 0);

		glTexCoord2f(1.0, 0.0);
		glVertex3f(15.0, -15.0, 15.0);
		glNormal3f(-1, 0, 0);

		glTexCoord2f(1.0, 1.0);
		glVertex3f(15.0, 15.0, 15.0);
		glNormal3f(-1, 0, 0);

		glTexCoord2f(0.0, 1.0);
		glVertex3f(15.0, 15.0, -15.0);
		glNormal3f(-1, 0, 0);
	glEnd();
	

	// Draw Left wall
	glBindTexture(GL_TEXTURE_2D, tex3);
	glBegin(GL_POLYGON);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(-15, -15, 15);
		glNormal3f(1, 0, 0);
		
		glTexCoord2f(1.0, 0.0);
		glVertex3f(-15, -15, -15);
		glNormal3f(1, 0, 0);
		
		glTexCoord2f(1.0, 1.0);
		glVertex3f(-15, 15, -15);
		glNormal3f(1, 0, 0);
		
		glTexCoord2f(0.0, 1.0);
		glVertex3f(-15, 15, 15);
		glNormal3f(1, 0, 0);	
	glEnd();


	// Draw Ceiling
	// The following two function calls glEnable(GL_BLEND) and glBlendFunc(...) enables transparency
	// Transparency is enabled until glDisable(GL_BLEND) is called
	// Not all textures have transparency enabled. Only transparent textures of file extension .png, .tga have transparency enabled
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glBindTexture(GL_TEXTURE_2D, tex4);
	glBegin(GL_POLYGON);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(-15, 15, 15);
		glNormal3f(1, 0, 0);
		
		glTexCoord2f(1.0, 0.0);
		glVertex3f(15, 15, 15);
		glNormal3f(1, 0, 0);
		
		glTexCoord2f(1.0, 1.0);
		glVertex3f(15, 15, -15);
		glNormal3f(1, 0, 0);
		
		glTexCoord2f(0.0, 1.0);
		glVertex3f(-15, 15, -15);
		glNormal3f(1, 0, 0);	
	glEnd();
	glDisable(GL_BLEND);

	
	glDisable(GL_TEXTURE_2D);
}



// this is for clamping the numbers between 0 & 360. used for rotation values in the mouse move function
void Render::clamp(float v0, float v1, float v2)
{
	if (v0 > 360 || v0 < -360)
		v0 = 0;
	if (v1 > 360 || v1 < -360)
		v1 = 0;
	if (v2 > 360 || v2 < -360)
		v2 = 0;
}

